@@ -60,11 +60,8 @@ namespace ranges
             constexpr auto& emplaced_index = static_const<emplaced_index_t<I>>::value;
         }
     #else // RANGES_CXX_INLINE_VARIABLES >= RANGES_CXX_INLINE_VARIABLES_17
-        inline namespace function_objects
-        {
-            template<std::size_t I>
-            inline constexpr emplaced_index_t<I> emplaced_index{};
-        }
+        template<std::size_t I>
+        inline constexpr emplaced_index_t<I> emplaced_index{};
     #endif  // RANGES_CXX_INLINE_VARIABLES
 
         /// \cond
@@ -394,14 +391,14 @@ namespace ranges
                 }
                 template<typename U>
                 meta::if_<std::is_object<U>>
-                    operator()(indexed_datum<U, meta::size_t<N>> &u)
+                operator()(indexed_datum<U, meta::size_t<N>> &u)
                     noexcept(std::is_nothrow_constructible<U, Ts...>::value)
                 {
                     this->construct_(u.get(), meta::make_index_sequence<sizeof...(Ts)>{});
                 }
                 template<typename U>
                 meta::if_<meta::not_<std::is_object<U>>>
-                    operator()(indexed_datum<U, meta::size_t<N>> &u)
+                operator()(indexed_datum<U, meta::size_t<N>> &u)
                     noexcept(std::is_nothrow_constructible<detail::decay_t<U>, Ts...>::value)
                 {
                     this->construct_(u, meta::make_index_sequence<sizeof...(Ts)>{});
@@ -423,6 +420,12 @@ namespace ranges
                 {
                     *t_ = std::addressof(t.get());
                 }
+                template<typename U>
+                void operator()(indexed_element<U &&, N> t) const noexcept
+                {
+                    U &&u = t.get();
+                    *t_ = std::addressof(u);
+                }
                 void operator()(indexed_element<void, N>) const noexcept
                 {}
             };
@@ -525,10 +528,10 @@ namespace ranges
           , private detail::variant_base<variant<Ts...>>
         {
         private:
-            friend struct detail::variant_core_access;
+            friend detail::variant_core_access;
             template<typename...>
             friend struct variant;
-            friend struct detail::variant_base<variant, false>;
+            friend detail::variant_base<variant, false>;
             template<std::size_t Index>
             using datum_t =
                 detail::variant_datum_t<Index, Ts...>;
