@@ -35,6 +35,10 @@ namespace ranges
         /// @{
         namespace adl_advance_detail
         {
+#ifdef RANGES_WORKAROUND_MSVC_620035
+            void advance();
+#endif
+
             template<typename I, typename D>
             void advance(I&, D) = delete;
 
@@ -209,6 +213,7 @@ namespace ranges
                 Concept)
             {
                 RANGES_EXPECT((Same<I, S>() || 0 <= n));
+                if (n == 0) return 0;
                 D d = bound - it;
                 RANGES_EXPECT(0 <= n ? 0 <= d : 0 >= d);
                 if(0 <= n ? n >= d : n <= d)
@@ -265,7 +270,14 @@ namespace ranges
             template<typename I,
                 CONCEPT_REQUIRES_(BidirectionalIterator<I>())>
             RANGES_CXX14_CONSTEXPR
-            I operator()(I it, difference_type_t<I> n = 1) const
+            I operator()(I it) const
+            {
+                return --it;
+            }
+            template<typename I,
+                CONCEPT_REQUIRES_(BidirectionalIterator<I>())>
+            RANGES_CXX14_CONSTEXPR
+            I operator()(I it, difference_type_t<I> n) const
             {
                 advance(it, -n);
                 return it;
@@ -608,6 +620,66 @@ namespace ranges
             Char const *delim_;
         };
 
+        template <typename Delim, typename Char = char,
+                  typename Traits = std::char_traits<Char>>
+        struct ostream_joiner
+        {
+            CONCEPT_ASSERT(SemiRegular<Delim>());
+            using difference_type = std::ptrdiff_t;
+            using char_type       = Char;
+            using traits_type     = Traits;
+            using ostream_type    = std::basic_ostream<Char, Traits>;
+
+            constexpr ostream_joiner() = default;
+            ostream_joiner(ostream_type &s, Delim const &d)
+              : delim_(d), sout_(std::addressof(s)), first_(true)
+            {}
+            ostream_joiner(ostream_type &s, Delim &&d)
+              : delim_(std::move(d)), sout_(std::addressof(s)), first_(true)
+            {}
+            template <typename T>
+            ostream_joiner& operator=(T const &value)
+            {
+                RANGES_EXPECT(sout_);
+                if (!first_)
+                    *sout_ << delim_;
+                first_ = false;
+                *sout_ << value;
+                return *this;
+            }
+            ostream_joiner& operator*() noexcept
+            {
+                return *this;
+            }
+            ostream_joiner& operator++() noexcept
+            {
+                return *this;
+            }
+            ostream_joiner& operator++(int) noexcept
+            {
+                return *this;
+            }
+          private:
+            Delim delim_;
+            ostream_type *sout_;
+            bool first_;
+        };
+
+        struct make_ostream_joiner_fn
+        {
+            template <typename Delim, typename Char, typename Traits,
+                CONCEPT_REQUIRES_(SemiRegular<detail::decay_t<Delim>>())>
+            ostream_joiner<detail::decay_t<Delim>, Char, Traits>
+            operator()(std::basic_ostream<Char, Traits> &s, Delim &&d) const
+            {
+                return {s, std::forward<Delim>(d)};
+            }
+        };
+
+        /// \ingroup group-utility
+        /// \sa `make_ostream_joiner_fn`
+        RANGES_INLINE_VARIABLE(make_ostream_joiner_fn, make_ostream_joiner)
+
         template<typename Char, typename Traits = std::char_traits<Char>>
         struct ostreambuf_iterator
         {
@@ -749,12 +821,17 @@ namespace ranges
         }  // namespace detail
         /// \endcond
 
-        template<typename I>
-        RANGES_CXX14_CONSTEXPR
-        reverse_iterator<I> make_reverse_iterator(I i)
+        struct make_reverse_iterator_fn
         {
-            return reverse_iterator<I>(i);
-        }
+            template<typename I>
+            RANGES_CXX14_CONSTEXPR
+            reverse_iterator<I> operator()(I i) const
+            {
+                return reverse_iterator<I>(i);
+            }
+        };
+
+        RANGES_INLINE_VARIABLE(make_reverse_iterator_fn, make_reverse_iterator)
 
         template<typename I>
         struct move_iterator
