@@ -1,7 +1,7 @@
 /// \file
 // Range v3 library
 //
-//  Copyright Eric Niebler 2014
+//  Copyright Eric Niebler 2014-present
 //
 //  Use, modification and distribution is subject to the
 //  Boost Software License, Version 1.0. (See accompanying
@@ -15,14 +15,15 @@
 
 #include <iosfwd>
 #include <meta/meta.hpp>
-#include <range/v3/range_fwd.hpp>
-#include <range/v3/range_concepts.hpp>
-#include <range/v3/range_traits.hpp>
 #include <range/v3/begin_end.hpp>
+#include <range/v3/empty.hpp>
+#include <range/v3/range_concepts.hpp>
+#include <range/v3/range_fwd.hpp>
+#include <range/v3/range_traits.hpp>
 #include <range/v3/to_container.hpp>
+#include <range/v3/utility/common_iterator.hpp>
 #include <range/v3/utility/concepts.hpp>
 #include <range/v3/utility/iterator.hpp>
-#include <range/v3/utility/common_iterator.hpp>
 
 namespace ranges
 {
@@ -38,7 +39,7 @@ namespace ranges
                 To to;
                 template<typename F, typename T,
                     CONCEPT_REQUIRES_(ConvertibleTo<F, From>() && ConvertibleTo<T, To>())>
-                slice_bounds(F from, T to)
+                constexpr slice_bounds(F from, T to)
                   : from(from), to(to)
                 {}
             };
@@ -50,7 +51,7 @@ namespace ranges
 
                 template<typename Other,
                     CONCEPT_REQUIRES_(Integral<Other>() && ExplicitlyConvertibleTo<Other, Int>())>
-                operator from_end_<Other> () const
+                constexpr operator from_end_<Other> () const
                 {
                     return {static_cast<Other>(dist_)};
                 }
@@ -65,13 +66,13 @@ namespace ranges
           : basic_view<Cardinality>
         {
         protected:
-            Derived & derived()
+            RANGES_CXX14_CONSTEXPR Derived &derived() noexcept
             {
                 CONCEPT_ASSERT(DerivedFrom<Derived, view_interface>());
                 return static_cast<Derived &>(*this);
             }
             /// \overload
-            Derived const & derived() const
+            constexpr Derived const &derived() const noexcept
             {
                 CONCEPT_ASSERT(DerivedFrom<Derived, view_interface>());
                 return static_cast<Derived const &>(*this);
@@ -84,33 +85,68 @@ namespace ranges
             view_interface &operator=(view_interface &&) = default;
             view_interface &operator=(view_interface const &) = default;
             // A few ways of testing whether a range can be empty:
-            CONCEPT_REQUIRES(Cardinality >= 0)
-            constexpr bool empty() const
+            CONCEPT_REQUIRES(Cardinality >= 0 || Cardinality == infinite)
+            constexpr bool empty() const noexcept
             {
                 return Cardinality == 0;
             }
-            template<class D = Derived,
-                CONCEPT_REQUIRES_(Same<D, Derived>() && Cardinality < 0 && ForwardRange<D const>())>
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(
+                    Cardinality < 0 && Cardinality != infinite &&
+                    !ForwardRange<D>() &&
+                    SizedRange<D>())>
+            RANGES_CXX14_CONSTEXPR bool empty()
+                noexcept(noexcept(bool(ranges::size(std::declval<D &>()) == 0)))
+            {
+                return ranges::size(derived()) == 0;
+            }
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(
+                    Cardinality < 0 && Cardinality != infinite &&
+                    !ForwardRange<D const>() &&
+                    SizedRange<D const>())>
             constexpr bool empty() const
+                noexcept(noexcept(bool(ranges::size(std::declval<D const &>()) == 0)))
             {
-                return derived().begin() == derived().end();
+                return ranges::size(derived()) == 0;
             }
-            template<class D = Derived,
-                CONCEPT_REQUIRES_(Same<D, Derived>() && (Cardinality >= 0 || ForwardRange<D const>()))>
-            constexpr bool operator!() const
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() &&
+                    Cardinality < 0 && Cardinality != infinite &&
+                    ForwardRange<D>())>
+            RANGES_CXX14_CONSTEXPR bool empty()
+                noexcept(noexcept(bool(ranges::begin(std::declval<D &>()) ==
+                    ranges::end(std::declval<D &>()))))
             {
-                return empty();
+                return ranges::begin(derived()) == ranges::end(derived());
             }
-            template<class D = Derived,
-                CONCEPT_REQUIRES_(Same<D, Derived>() && (Cardinality >= 0 || ForwardRange<D const>()))>
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() &&
+                    Cardinality < 0 && Cardinality != infinite &&
+                    ForwardRange<D const>())>
+            constexpr bool empty() const
+                noexcept(noexcept(bool(ranges::begin(std::declval<D const &>()) ==
+                    ranges::end(std::declval<D const &>()))))
+            {
+                return ranges::begin(derived()) == ranges::end(derived());
+            }
+            template<typename D = Derived, CONCEPT_REQUIRES_(Same<D, Derived>())>
+            constexpr auto operator!() const
+            RANGES_DECLTYPE_NOEXCEPT(ranges::empty(std::declval<D const &>()))
+            {
+                return ranges::empty(derived());
+            }
+            template<typename D = Derived, CONCEPT_REQUIRES_(Same<D, Derived>()),
+                typename = decltype(ranges::empty(std::declval<D const &>()))>
             constexpr explicit operator bool() const
+                noexcept(noexcept(ranges::empty(std::declval<D const &>())))
             {
-                return !empty();
+                return !ranges::empty(derived());
             }
             /// Access the size of the range, if it can be determined:
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && Cardinality >= 0)>
-            constexpr range_size_type_t<D> size() const
+            constexpr range_size_type_t<D> size() const noexcept
             {
                 return (range_size_type_t<D>)Cardinality;
             }
@@ -122,38 +158,46 @@ namespace ranges
             {
                 return iter_size(derived().begin(), derived().end());
             }
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && Cardinality < 0 &&
+                    SizedSentinel<sentinel_t<D>, iterator_t<D>>() &&
+                    ForwardRange<D>())>
+            RANGES_CXX14_CONSTEXPR range_size_type_t<D> size()
+            {
+                return iter_size(derived().begin(), derived().end());
+            }
             /// Access the first element in a range:
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && ForwardRange<D>())>
-            range_reference_t<D> front()
+            RANGES_CXX14_CONSTEXPR range_reference_t<D> front()
             {
                 return *derived().begin();
             }
             /// \overload
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && ForwardRange<D const>())>
-            range_reference_t<D const> front() const
+            constexpr range_reference_t<D const> front() const
             {
                 return *derived().begin();
             }
             /// Access the last element in a range:
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && BoundedRange<D>() && BidirectionalRange<D>())>
-            range_reference_t<D> back()
+            RANGES_CXX14_CONSTEXPR range_reference_t<D> back()
             {
                 return *prev(derived().end());
             }
             /// \overload
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && BoundedRange<D const>() && BidirectionalRange<D const>())>
-            range_reference_t<D const> back() const
+            constexpr range_reference_t<D const> back() const
             {
                 return *prev(derived().end());
             }
             /// Simple indexing:
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && RandomAccessRange<D>())>
-            auto operator[](range_difference_type_t<D> n) ->
+            RANGES_CXX14_CONSTEXPR auto operator[](range_difference_type_t<D> n) ->
                 decltype(std::declval<D &>().begin()[n])
             {
                 return derived().begin()[n];
@@ -161,7 +205,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived,
                 CONCEPT_REQUIRES_(Same<D, Derived>() && RandomAccessRange<D const>())>
-            auto operator[](range_difference_type_t<D> n) const ->
+            constexpr auto operator[](range_difference_type_t<D> n) const ->
                 decltype(std::declval<D const &>().begin()[n])
             {
                 return derived().begin()[n];
@@ -170,6 +214,7 @@ namespace ranges
             //      rng[{4,6}]
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>> offs) & ->
                 decltype(std::declval<Slice>()(std::declval<D &>(), offs.from, offs.to))
             {
@@ -178,6 +223,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            constexpr
             auto operator[](detail::slice_bounds<range_difference_type_t<D>> offs) const & ->
                 decltype(std::declval<Slice>()(std::declval<D const &>(), offs.from, offs.to))
             {
@@ -186,6 +232,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>> offs) && ->
                 decltype(std::declval<Slice>()(std::declval<D>(), offs.from, offs.to))
             {
@@ -195,6 +242,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>,
                 detail::from_end_<range_difference_type_t<D>>> offs) & ->
                 decltype(std::declval<Slice>()(std::declval<D &>(), offs.from, offs.to))
@@ -204,7 +252,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
-            auto operator[](detail::slice_bounds<range_difference_type_t<D>,
+            constexpr auto operator[](detail::slice_bounds<range_difference_type_t<D>,
                 detail::from_end_<range_difference_type_t<D>>> offs) const & ->
                 decltype(std::declval<Slice>()(std::declval<D const &>(), offs.from, offs.to))
             {
@@ -213,6 +261,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>,
                 detail::from_end_<range_difference_type_t<D>>> offs) && ->
                 decltype(std::declval<Slice>()(std::declval<D>(), offs.from, offs.to))
@@ -223,6 +272,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>,
                 detail::from_end_<range_difference_type_t<D>>> offs) & ->
                 decltype(std::declval<Slice>()(std::declval<D &>(), offs.from, offs.to))
@@ -232,6 +282,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            constexpr
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>,
                 detail::from_end_<range_difference_type_t<D>>> offs) const & ->
                 decltype(std::declval<Slice>()(std::declval<D const &>(), offs.from, offs.to))
@@ -241,6 +292,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>,
                 detail::from_end_<range_difference_type_t<D>>> offs) && ->
                 decltype(std::declval<Slice>()(std::declval<D>(), offs.from, offs.to))
@@ -251,6 +303,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>, end_fn> offs) & ->
                 decltype(std::declval<Slice>()(std::declval<D &>(), offs.from, offs.to))
             {
@@ -259,6 +312,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            constexpr
             auto operator[](detail::slice_bounds<range_difference_type_t<D>, end_fn> offs) const & ->
                 decltype(std::declval<Slice>()(std::declval<D const &>(), offs.from, offs.to))
             {
@@ -267,6 +321,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<range_difference_type_t<D>, end_fn> offs) && ->
                 decltype(std::declval<Slice>()(std::declval<D>(), offs.from, offs.to))
             {
@@ -276,6 +331,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>, end_fn> offs) & ->
                 decltype(std::declval<Slice>()(std::declval<D &>(), offs.from, offs.to))
             {
@@ -284,6 +340,7 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            constexpr
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>, end_fn> offs) const & ->
                 decltype(std::declval<Slice>()(std::declval<D const &>(), offs.from, offs.to))
             {
@@ -292,16 +349,43 @@ namespace ranges
             /// \overload
             template<typename D = Derived, typename Slice = view::slice_fn,
                 CONCEPT_REQUIRES_(Same<D, Derived>())>
+            RANGES_CXX14_CONSTEXPR
             auto operator[](detail::slice_bounds<detail::from_end_<range_difference_type_t<D>>, end_fn> offs) && ->
                 decltype(std::declval<Slice>()(std::declval<D>(), offs.from, offs.to))
             {
                 return Slice{}(detail::move(derived()), offs.from, offs.to);
             }
+            /// Returns a reference to the element at specified location pos, with bounds checking.
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && RandomAccessRange<D>() && SizedRange<D>())>
+            RANGES_CXX14_CONSTEXPR auto at(range_difference_type_t<D> n) ->
+                decltype(std::declval<D &>().begin()[n])
+            {
+                using size_type = range_size_type_t<Derived>;
+                if (n < 0 || size_type(n) >= ranges::size(derived()))
+                {
+                    throw std::out_of_range("view_interface::at");
+                }
+                return derived().begin()[n];
+            }
+            /// \overload
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && RandomAccessRange<D const>() && SizedRange<D const>())>
+            RANGES_CXX14_CONSTEXPR auto at(range_difference_type_t<D> n) const  ->
+                decltype(std::declval<D const &>().begin()[n])
+            {
+                using size_type = range_size_type_t<Derived>;
+                if (n < 0 || size_type(n) >= ranges::size(derived()))
+                {
+                    throw std::out_of_range("view_interface::at");
+                }
+                return derived().begin()[n];
+            }
             /// Implicit conversion to something that looks like a container.
             template<typename Container, typename D = Derived,
                 typename = typename Container::allocator_type, // HACKHACK
                 CONCEPT_REQUIRES_(detail::ConvertibleToContainer<D, Container>())>
-            operator Container ()
+            RANGES_CXX14_CONSTEXPR operator Container ()
             {
                 return ranges::to_<Container>(derived());
             }
@@ -309,48 +393,51 @@ namespace ranges
             template<typename Container, typename D = Derived,
                 typename = typename Container::allocator_type, // HACKHACK
                 CONCEPT_REQUIRES_(detail::ConvertibleToContainer<D const, Container>())>
-            operator Container () const
+            constexpr operator Container () const
             {
                 return ranges::to_<Container>(derived());
             }
             /// \brief Print a range to an ostream
-            template<bool B = true, typename Stream = meta::if_c<B, std::ostream>>
-            friend Stream &operator<<(Stream &sout, Derived &rng)
+        private:
+            template<typename Stream, typename Rng,
+                CONCEPT_REQUIRES_(Same<Derived, meta::_t<std::remove_cv<Rng>>>())>
+            static Stream &print_(Stream &sout, Rng &rng)
             {
+                sout << '[';
                 auto it = ranges::begin(rng);
                 auto const e = ranges::end(rng);
-                if(it == e)
-                    return sout << "[]";
-                sout << '[' << *it;
-                while(++it != e)
-                    sout << ',' << *it;
-                sout << ']';
-                return sout;
-            }
-            /// \overload
-            template<bool B = true, typename Stream = meta::if_c<B, std::ostream>,
-                typename D = Derived, CONCEPT_REQUIRES_(InputRange<D const>())>
-            friend Stream &operator<<(Stream &sout, Derived const &rng)
-            {
-                auto it = ranges::begin(rng);
-                auto const e = ranges::end(rng);
-                if(it == e)
+                if(it != e)
                 {
-                    sout << "[]";
-                    return sout;
+                    for(;;)
+                    {
+                        sout << *it;
+                        if(++it == e) break;
+                        sout << ',';
+                    }
                 }
-                sout << '[' << *it;
-                while(++it != e)
-                    sout << ',' << *it;
                 sout << ']';
                 return sout;
             }
-            /// \overload
-            template<bool B = true, typename Stream = meta::if_c<B, std::ostream>>
-            friend Stream &operator<<(Stream &sout, Derived &&rng)
+
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && InputRange<D const>())>
+            friend std::ostream &operator<<(std::ostream &sout, Derived const &rng)
             {
-                sout << rng;
-                return sout;
+                return view_interface::print_(sout, rng);
+            }
+            /// \overload
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && !Range<D const>() && InputRange<D>())>
+            friend std::ostream &operator<<(std::ostream &sout, Derived &rng)
+            {
+                return view_interface::print_(sout, rng);
+            }
+            /// \overload
+            template<typename D = Derived,
+                CONCEPT_REQUIRES_(Same<D, Derived>() && !Range<D const>() && InputRange<D>())>
+            friend std::ostream &operator<<(std::ostream &sout, Derived &&rng)
+            {
+                return view_interface::print_(sout, rng);
             }
         };
         /// @}
